

library IEEE;
use ieee.std_logic_1164.ALL;
use ieee.numeric_std.all;

entity spartan3AN_test is
    port(
        CLK_50MHZ : in std_logic;
        SPI_SCK : out std_logic;
        SPI_MOSI : out std_logic;
        DAC_CLR : out std_logic;
        DAC_CS : out std_logic;
        btn_south : in std_logic
    );
end entity spartan3AN_test;

architecture rtl of spartan3AN_test is

	type dacStateType is (
		idle,
		sendBit,
		clockHigh,
		csHigh
	);
	signal dacState   : dacStateType := idle;	
	-- signal dacData : unsigned(31 downto 0);
	signal dacData : unsigned(23 downto 0);
	signal dacCounter : integer range dacData'range;
	-- signal output : unsigned(11 downto 0);
	signal output : unsigned(7 downto 0) := "00000000";	
	constant counter_value : unsigned(7 downto 0) := "00000010";
	signal counter : unsigned(7 downto 0) := counter_value;
	type states is (change, waitstate, test);
	signal state : states;
	type newstates is (count_up, count_down);
	signal newstate : newstates;
 
begin
 
	-- Dreiecksspannung
   process (CLK_50MHz)
	begin
	  if rising_edge(CLK_50MHZ)
	  then
		if btn_south = '1'
		then
			output <= (others => '0');
		else
			case state is
				when change => 	if newstate = count_up
										then 
											output <= output + 1;
										else
											output <= output - 1;
										end if;
										state <= waitstate;
				when waitstate => counter <= counter - 1;
										if counter = 1
										then 
											state <= test;
										end if;
				when test =>		counter <= counter_value;
										if output = "11111111"
										then
											newstate <= count_down; 
										elsif output = "00000000"
										then
											newstate <= count_up;
										end if;
										state <= change;
			end case;
		end if;
	  end if;
	end process;	
		
		
	process(CLK_50MHZ, btn_south)
    begin
        if rising_edge(CLK_50MHZ)
        then		  
			  if btn_south = '1' 
			  then
            dacState <= idle;
			 else        			  
						case dacState is
							 when idle =>
									DAC_CS <= '0';
									SPI_SCK <= '0';									
									dacCounter <= dacData'high;
									--dacData <= "00000000" & "0010" & "1111" & output & "0000";
									dacData <= "0010" & "1111" & output & "00000000";
									dacState <= sendBit;
							 when sendBit =>
								  SPI_SCK <= '0';
								  SPI_MOSI <= dacData(dacData'high);
								  dacData <= dacData(dacData'high-1 downto dacData'low) & "0";
								  dacState <= clockHigh;
							 when clockHigh =>
								  SPI_SCK <= '1';
								  if dacCounter = dacData'low then
										dacState <= csHigh;
								  else
										dacCounter <= dacCounter - 1;
										dacState <= sendBit;
								  end if;
							 when csHigh =>
								  DAC_CS <= '1';
								  dacState <= idle;
						end case;
        end if;
		 end if;
    end process;

    DAC_CLR <= '1';
	
end architecture rtl;